@@ -1,19 +1,10 @@
 // Copyright (C) 2020 Mathieu Duponchelle <mathieu@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
+// SPDX-License-Identifier: MPL-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -62,36 +53,8 @@ impl Default for Settings {
     }
 }
 
-// FIXME: https://github.com/mgeisler/textwrap/issues/412
-#[derive(Debug)]
-struct WrappedWordSplitter(Box<dyn textwrap::word_splitters::WordSplitter + Send>);
-
-impl textwrap::word_splitters::WordSplitter for WrappedWordSplitter {
-    fn split_points(&self, word: &str) -> Vec<usize> {
-        self.0.split_points(word)
-    }
-}
-
-impl Clone for WrappedWordSplitter {
-    fn clone(&self) -> Self {
-        WrappedWordSplitter(unsafe {
-            std::mem::transmute::<
-                Box<dyn textwrap::word_splitters::WordSplitter + 'static>,
-                Box<dyn textwrap::word_splitters::WordSplitter + Send + 'static>,
-            >(self.0.clone_box())
-        })
-    }
-}
-
 struct State {
-    options: Option<
-        textwrap::Options<
-            'static,
-            textwrap::wrap_algorithms::OptimalFit,
-            textwrap::word_separators::UnicodeBreakProperties,
-            WrappedWordSplitter,
-        >,
-    >,
+    options: Option<textwrap::Options<'static>>,
 
     current_text: String,
     start_ts: Option<gst::ClockTime>,
@@ -130,7 +93,9 @@ impl TextWrap {
             return;
         }
 
-        state.options = if let Some(dictionary) = &settings.dictionary {
+        let mut options = textwrap::Options::new(settings.columns as usize);
+
+        if let Some(dictionary) = &settings.dictionary {
             let dict_file = match File::open(dictionary) {
                 Err(err) => {
                     gst_error!(CAT, obj: element, "Failed to open dictionary file: {}", err);
@@ -153,16 +118,12 @@ impl TextWrap {
                 Ok(standard) => standard,
             };
 
-            Some(textwrap::Options::with_word_splitter(
-                settings.columns as usize,
-                WrappedWordSplitter(Box::new(standard)),
-            ))
+            options.word_splitter = textwrap::WordSplitter::Hyphenation(standard);
         } else {
-            Some(textwrap::Options::with_word_splitter(
-                settings.columns as usize,
-                WrappedWordSplitter(Box::new(textwrap::word_splitters::NoHyphenation)),
-            ))
-        };
+            options.word_splitter = textwrap::WordSplitter::NoHyphenation;
+        }
+
+        state.options = Some(options);
     }
 
     fn sink_chain(
@@ -201,24 +162,20 @@ impl TextWrap {
             let mut bufferlist = gst::BufferList::new();
             let n_lines = std::cmp::max(self.settings.lock().unwrap().lines, 1);
 
-            if state
+            let add_buffer = state
                 .start_ts
-                .zip(Some(accumulate_time))
-                .map_or(false, |(start_ts, accumulate_time)| {
-                    start_ts + accumulate_time < pts
-                })
-            {
+                .opt_add(accumulate_time)
+                .opt_lt(pts)
+                .unwrap_or(false);
+
+            if add_buffer {
                 let mut buf =
                     gst::Buffer::from_mut_slice(mem::take(&mut state.current_text).into_bytes());
                 {
                     let buf_mut = buf.get_mut().unwrap();
                     buf_mut.set_pts(state.start_ts);
-                    buf_mut.set_duration(
-                        state
-                            .end_ts
-                            .zip(state.start_ts)
-                            .and_then(|(end_ts, start_ts)| end_ts.checked_sub(start_ts)),
-                    );
+                    buf_mut
+                        .set_duration(state.end_ts.opt_checked_sub(state.start_ts).ok().flatten());
                 }
                 bufferlist.get_mut().unwrap().add(buf);
 
@@ -261,10 +218,7 @@ impl TextWrap {
                             .collect::<Vec<String>>()
                             .join("\n");
                     } else {
-                        let duration = state
-                            .end_ts
-                            .zip(state.start_ts)
-                            .and_then(|(end_ts, start_ts)| end_ts.checked_sub(start_ts));
+                        let duration = state.end_ts.opt_checked_sub(state.start_ts).ok().flatten();
                         let contents = chunk
                             .iter()
                             .map(|l| l.to_string())
@@ -290,7 +244,7 @@ impl TextWrap {
                 }
 
                 current_text = trailing;
-                state.end_ts = state.end_ts.map(|end_ts| end_ts + duration_per_word);
+                state.end_ts = state.end_ts.opt_add(duration_per_word);
             }
 
             state.current_text = current_text;
@@ -398,10 +352,7 @@ impl TextWrap {
                         let buf_mut = buf.get_mut().unwrap();
                         buf_mut.set_pts(state.start_ts);
                         buf_mut.set_duration(
-                            state
-                                .end_ts
-                                .zip(state.start_ts)
-                                .and_then(|(end_ts, start_ts)| end_ts.checked_sub(start_ts)),
+                            state.end_ts.opt_checked_sub(state.start_ts).ok().flatten(),
                         );
                     }
 
@@ -508,7 +459,7 @@ impl ObjectImpl for TextWrap {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
             vec![
-                glib::ParamSpec::new_string(
+                glib::ParamSpecString::new(
                     "dictionary",
                     "Dictionary",
                     "Path to a dictionary to load at runtime to perform hyphenation, see \
@@ -516,7 +467,7 @@ impl ObjectImpl for TextWrap {
                     None,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "columns",
                     "Columns",
                     "Maximum number of columns for any given line",
@@ -525,7 +476,7 @@ impl ObjectImpl for TextWrap {
                     DEFAULT_COLUMNS,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_uint(
+                glib::ParamSpecUInt::new(
                     "lines",
                     "Lines",
                     "Split input buffer into output buffers with max lines (0=do not split)",
@@ -534,7 +485,7 @@ impl ObjectImpl for TextWrap {
                     DEFAULT_LINES,
                     glib::ParamFlags::READWRITE | gst::PARAM_FLAG_MUTABLE_PLAYING,
                 ),
-                glib::ParamSpec::new_uint64(
+                glib::ParamSpecUInt64::new(
                     "accumulate-time",
                     "accumulate-time",
                     "Cut-off time for input text accumulation (0=do not accumulate)",
@@ -623,6 +574,8 @@ impl ObjectImpl for TextWrap {
     }
 }
 
+impl GstObjectImpl for TextWrap {}
+
 impl ElementImpl for TextWrap {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -640,7 +593,7 @@ impl ElementImpl for TextWrap {
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: Lazy<Vec<gst::PadTemplate>> = Lazy::new(|| {
             let caps = gst::Caps::builder("text/x-raw")
-                .field("format", &"utf8")
+                .field("format", "utf8")
                 .build();
             let src_pad_template = gst::PadTemplate::new(
                 "src",
